use std::io;

use axum::{
    routing::{get, get_service},
    extract::Path,
    http::{StatusCode},
    response,
    Router};

async fn hello(Path(name): Path<String>) -> impl response::IntoResponse {
    (StatusCode::OK, format!("Hello {}", name))
}

#[tokio::main]
async fn main() {
    tracing_subscriber::fmt::init();
    let app = Router::new()
        .route("/hello/:name", get(hello))
        .route("/static",
               get_service(tower_http::services::ServeFile::new("Cargo.toml").precompressed_gzip())
                   .handle_error(|e: io::Error| async move {
                       (StatusCode::INTERNAL_SERVER_ERROR, format!("Unhandled internal error: {}", e))
                   })
        )
        .layer(
            tower::ServiceBuilder::new()
                .layer(tower_http::trace::TraceLayer::new_for_http())
        );

    let addr = std::net::SocketAddr::from(([127, 0, 0, 1], 3000));
    tracing::info!("listening on {}", addr);
    axum::Server::bind(&addr)
        .serve(app.into_make_service())
        .await
        .unwrap();
}
